@@ -93,7 +93,9 @@ def eliminate_qualify(expression: exp.Expression) -> exp.Expression:
     Some dialects don't support window functions in the WHERE clause, so we need to include them as
     projections in the subquery, in order to refer to them in the outer filter using aliases. Also,
     if a column is referenced in the QUALIFY clause but is not selected, we need to include it too,
-    otherwise we won't be able to refer to it in the outer query's WHERE clause.
+    otherwise we won't be able to refer to it in the outer query's WHERE clause. Finally, if a
+    newly aliased projection is referenced in the QUALIFY clause, it will be replaced by the
+    corresponding expression to avoid creating invalid column references.
     """
     if isinstance(expression, exp.Select) and expression.args.get("qualify"):
         taken = set(expression.named_selects)
@@ -105,20 +107,31 @@ def eliminate_qualify(expression: exp.Expression) -> exp.Expression:
 
         outer_selects = exp.select(*[select.alias_or_name for select in expression.selects])
         qualify_filters = expression.args["qualify"].pop().this
+        expression_by_alias = {
+            select.alias: select.this
+            for select in expression.selects
+            if isinstance(select, exp.Alias)
+        }
 
         select_candidates = exp.Window if expression.is_star else (exp.Window, exp.Column)
-        for expr in qualify_filters.find_all(select_candidates):
-            if isinstance(expr, exp.Window):
+        for select_candidate in qualify_filters.find_all(select_candidates):
+            if isinstance(select_candidate, exp.Window):
+                if expression_by_alias:
+                    for column in select_candidate.find_all(exp.Column):
+                        expr = expression_by_alias.get(column.name)
+                        if expr:
+                            column.replace(expr)
+
                 alias = find_new_name(expression.named_selects, "_w")
-                expression.select(exp.alias_(expr, alias), copy=False)
+                expression.select(exp.alias_(select_candidate, alias), copy=False)
                 column = exp.column(alias)
 
-                if isinstance(expr.parent, exp.Qualify):
+                if isinstance(select_candidate.parent, exp.Qualify):
                     qualify_filters = column
                 else:
-                    expr.replace(column)
-            elif expr.name not in expression.named_selects:
-                expression.select(expr.copy(), copy=False)
+                    select_candidate.replace(column)
+            elif select_candidate.name not in expression.named_selects:
+                expression.select(select_candidate.copy(), copy=False)
 
         return outer_selects.from_(expression.subquery(alias="_t", copy=False), copy=False).where(
             qualify_filters, copy=False
@@ -140,6 +153,26 @@ def remove_precision_parameterized_types(expression: exp.Expression) -> exp.Expr
     return expression
 
 
+def unqualify_unnest(expression: exp.Expression) -> exp.Expression:
+    """Remove references to unnest table aliases, added by the optimizer's qualify_columns step."""
+    from sqlglot.optimizer.scope import find_all_in_scope
+
+    if isinstance(expression, exp.Select):
+        unnest_aliases = {
+            unnest.alias
+            for unnest in find_all_in_scope(expression, exp.Unnest)
+            if isinstance(unnest.parent, (exp.From, exp.Join))
+        }
+        if unnest_aliases:
+            for column in expression.find_all(exp.Column):
+                if column.table in unnest_aliases:
+                    column.set("table", None)
+                elif column.db in unnest_aliases:
+                    column.set("db", None)
+
+    return expression
+
+
 def unnest_to_explode(expression: exp.Expression) -> exp.Expression:
     """Convert cross join unnest into lateral view explode."""
     if isinstance(expression, exp.Select):
@@ -316,13 +349,10 @@ def explode_to_unnest(index_offset: int = 0) -> t.Callable[[exp.Expression], exp
     return _explode_to_unnest
 
 
-PERCENTILES = (exp.PercentileCont, exp.PercentileDisc)
-
-
 def add_within_group_for_percentiles(expression: exp.Expression) -> exp.Expression:
     """Transforms percentiles by adding a WITHIN GROUP clause to them."""
     if (
-        isinstance(expression, PERCENTILES)
+        isinstance(expression, exp.PERCENTILES)
         and not isinstance(expression.parent, exp.WithinGroup)
         and expression.expression
     ):
@@ -338,7 +368,7 @@ def remove_within_group_for_percentiles(expression: exp.Expression) -> exp.Expre
     """Transforms percentiles by getting rid of their corresponding WITHIN GROUP clause."""
     if (
         isinstance(expression, exp.WithinGroup)
-        and isinstance(expression.this, PERCENTILES)
+        and isinstance(expression.this, exp.PERCENTILES)
         and isinstance(expression.expression, exp.Order)
     ):
         quantile = expression.this.this
@@ -447,7 +477,7 @@ def move_ctes_to_top_level(expression: exp.Expression) -> exp.Expression:
             if inner_with.recursive:
                 top_level_with.set("recursive", True)
 
-            top_level_with.expressions.extend(inner_with.expressions)
+            top_level_with.set("expressions", inner_with.expressions + top_level_with.expressions)
 
     return expression
 
@@ -464,7 +494,7 @@ def ensure_bools(expression: exp.Expression) -> exp.Expression:
         ):
             node.replace(node.neq(0))
 
-    for node, *_ in expression.walk():
+    for node in expression.walk():
         ensure_bools(node, _ensure_bool)
 
     return expression
@@ -561,9 +591,7 @@ def move_partitioned_by_to_schema_columns(expression: exp.Expression) -> exp.Exp
 
 
 def struct_kv_to_alias(expression: exp.Expression) -> exp.Expression:
-    """
-    Convert struct arguments to aliases: STRUCT(1 AS y) .
-    """
+    """Converts struct arguments to aliases, e.g. STRUCT(1 AS y)."""
     if isinstance(expression, exp.Struct):
         expression.set(
             "expressions",
